program Rectangles2Test;
  { test driver for unit Rectangles2 }

uses
  SysUtils,
  Rectangles2;
//Rectangles3;

procedure TestCreate(AXL, AYL, AXH, AYH: Integer; pre: Boolean);
  var
    r: TRectangle;
  begin
    writeln('Create(', AXL, ', ', AYL, ', ', AXH, ', ', AYH, ')')
  ; if pre then begin { precondition holds }
      try
        r := TRectangle.Create(AXL, AYL, AXH, AYH)
      ; writeln('  (OK) Created: ', r.ToStr)
      except
        on e : Exception do begin
          writeln('  FAILURE: Exception ', e.message)
        end
        else
          writeln('  FAILURE: Unrecognized exception')
      end { try }
    end
    else begin
      try
        r := TRectangle.Create(AXL, AYL, AXH, AYH)
      ; writeln('  FAILURE: EAssertionFailed expected but not caught')
      except
        on e : EAssertionFailed do begin
          writeln('  (OK) EAssertionFailed : ', e.message)
        end;
        on e : Exception do begin
          writeln('  FAILURE: Exception ', e.message)
        end
        else
          writeln('  FAILURE: Unrecognized exception')
      end { try }
    end
  end; { TestCreate }

procedure TestSet;
  var
    r: TRectangle;
  begin
    writeln('Testing Set...')
  ; r := TRectangle.Create(100, 100, 100, 100)
  ; with r do begin
      writeln('  r =       ', ToStr)
    ; writeln('  r.SetXH(200)')
    ; SetXH(200)
    ; writeln('    Effect: ', ToStr)
    ; writeln('  r.SetXL(150)')
    ; SetXL(150)
    ; writeln('    Effect: ', ToStr)
    ; writeln('  r.SetYH(301)')
    ; SetYH(301)
    ; writeln('    Effect: ', ToStr)
    ; writeln('  r.SetYL(252)')
    ; SetYL(252)
    ; writeln('    Effect: ', ToStr)
    end { with r }
  end; { TestSet }

procedure TestContains(AXL, AYL, AXH, AYH: Integer);
  { test all points in rectangle AXL-1, AYL-1, AXH+1, AYH+1 }
  { pre: AXL <= AXH  /\  AYL <= AYH }
  var
    r: TRectangle;
    x, y: Integer;
  begin
    writeln('Testing Contains...')
  ; r := TRectangle.Create(AXL, AYL, AXH, AYH)
  ; with r do begin
      writeln('  r =       ', ToStr)

    ; for x := AXL - 1 to AXH + 1 do begin

        for y := AYL - 1 to AYH + 1 do begin
          writeln('  r.Contains(', x, ', ', y, ') = ',
                  Contains(x, y))
        end { for y }

      end { for x }

    end { with r }
  end; { TestContains }

procedure TestIntersects(AXL, AYL, AXH, AYH, d: Integer);
  { test rectangle AXL, AYL, AXH, AYH against all relevant d-squares }
  { pre: AXL <= AXH  /\  AYL <= AYH }
  var
    r, s: TRectangle;
    x, y: Integer;
  begin
    writeln('Testing Intersects...')
  ; r := TRectangle.Create(AXL, AYL, AXH, AYH)
  ; with r do begin
      writeln('  r =          ', ToStr)

    ; for x := AXL - d to AXH + 1 do begin

        for y := AYL - d to AYH + 1 do begin
          s := TRectangle.Create(x, y, x + d - 1, y + d - 1)
        ; writeln('  r.Intersects(', s.ToStr, ') = ',
                  Intersects(s))
        end { for y }

      end { for x }

    end { with r }
  end; { TestIntersects }

procedure TestHull;
  const
    XX = 101;
    YY = 201;
  var
    r, s: TRectangle;
    x, y: Integer;
  begin
    writeln('Testing Hull...')

  ; for x := XX - 1 to XX + 1 do begin

      for y := YY - 1 to YY + 1 do begin
        r := TRectangle.Create(XX, YY, XX, YY)
      ; with r do begin
          writeln('  r =    ', ToStr)
        ; s := TRectangle.Create(x, y, x, y)
        ; r.Hull(s)
        ; writeln('  r.Hull(', s.ToStr, ') = ', ToStr)
        end { for y }

      end { for x }

    end { with r }
  end; { TestHull }

procedure TestIntersect(AXL, AYL, AXH, AYH, d: Integer);
  { intersect rectangle AXL, AYL, AXH, AYH with all d-squares }
  { pre: AXL <= AXH  /\  AYL <= AYH }
  var
    r, s: TRectangle;
    x, y: Integer;
  begin
    writeln('Testing Intersect...')

  ; for x := AXL - d + 1 to AXH do begin

      for y := AYL - d + 1 to AYH do begin
        r := TRectangle.Create(AXL, AYL, AXH, AYH)
      ; with r do begin
          writeln('  r =         ', ToStr)
        ; s := TRectangle.Create(x, y, x + d - 1, y + d - 1)
        ; r.Intersect(s)
        ; writeln('  r.Intersect(', s.ToStr, ') = ', ToStr)
        end { with r }
      end { for y }

    end { for x }

  end; { TestIntersect }

begin
  TestCreate(0, 0, 0, 0, True)
; TestCreate(1, 0, 0, 0, False)
; TestCreate(0, 1, 0, 0, False)

; TestSet
; TestContains(101, 202, 101, 202)
; TestContains(101, 204, 103, 206)
; TestIntersects(101, 205, 101, 205, 1)
; TestIntersects(110, 220, 113, 223, 2)
; TestHull
; TestIntersect(200, 304, 203, 307, 2)
end.
